@@ -70,15 +70,47 @@ std::string getServerName(void)
 
 bool getSettingsValue(std::wstring key, std::wstring *ret, CFileSettingsReader *settings)
 {
-	if(!settings->getValue(key, ret) )
+	std::wstring use_str;
+	settings->getValue(key + L".use", &use_str);
+	int use = watoi(use_str);
+
+	if (use == c_use_value_client)
 	{
-		if(!settings->getValue(key+L"_def", ret) )
+		if (settings->getValue(key + L".client", ret))
 		{
-			return false;
+			return true;
+		}
+	}
+	else if(use == c_use_value)
+	{
+		if (settings->getValue(key + L".home", ret))
+		{
+			return true;
+		}
+	}
+	else if (use == c_use_group)
+	{
+		if (settings->getValue(key + L".group", ret))
+		{
+			return true;
+		}
+	}
+	else
+	{
+		if (!settings->getValue(key, ret))
+		{
+			if (settings->getValue(key + L"_def", ret))
+			{
+				return true;
+			}
+		}
+		else
+		{
+			return true;
 		}
 	}
 
-	return true;
+	return false;
 }
 
 wxTextValidator getPathValidator(void)
@@ -118,7 +150,8 @@ wxTextValidator getDigitSlashValidator(void)
 	return val;
 }
 
-Settings::Settings(wxWindow* parent) : GUISettings(parent)
+Settings::Settings(wxWindow* parent) : GUISettings(parent),
+	init_complete(false)
 {
 	SetIcon(wxIcon(res_path+wxT("backup-ok.")+ico_ext, ico_type));
 #ifdef _DEBUG
@@ -129,6 +162,36 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	settings = new CFileSettingsReader(VARDIR "/urbackup/data/settings.cfg");
 #endif
 
+	std::wstring client_settings_tray_access_pw;
+	if (getSettingsValue(L"client_settings_tray_access_pw", &client_settings_tray_access_pw, settings)
+		&& !client_settings_tray_access_pw.empty())
+	{
+		do
+		{
+			wxPasswordEntryDialog* pwEntry = new wxPasswordEntryDialog(this, _("Please enter the tray access text"));
+			if (pwEntry->ShowModal() != wxID_OK)
+			{
+				Close();
+				return;
+			}
+
+			if (pwEntry->GetValue() == client_settings_tray_access_pw)
+			{
+				break;
+			}
+			else
+			{
+				wxDialog* pwWrong = new wxDialog(this, -1, _("Tray access text wrong. Please retry or cancel."));
+				if (pwWrong->ShowModal() != wxID_OK)
+				{
+					Close();
+					return;
+				}
+
+			}
+		} while (true);
+	}
+
 	std::wstring t;
 	if(getSettingsValue(L"update_freq_incr", &t, settings))
 	{
@@ -138,6 +201,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl1->SetValue(wxT("12"));
 	}
+	setSettingsSwitch(L"update_freq_incr", m_bitmapButton1, m_textCtrl1);
 	if(getSettingsValue(L"update_freq_full", &t, settings))
 	{
 		m_textCtrl2->SetValue(wxString(convert(watoi(t)/24/60/60).c_str()));
@@ -146,6 +210,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl2->SetValue(wxT("30"));
 	}
+	setSettingsSwitch(L"update_freq_full", m_bitmapButton2, m_textCtrl2);
 #ifdef _WIN32
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
 	{
@@ -169,6 +234,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 			m_textCtrl22->SetValue(wxT("60"));
 			m_checkBox1->SetValue(true);
 		}
+		setSettingsSwitch(L"update_freq_image_full", m_bitmapButton22, m_textCtrl22);
 		if(getSettingsValue(L"update_freq_image_incr", &t, settings))
 		{
 			if(watoi(t)>0)
@@ -180,6 +246,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 		{
 			m_textCtrl21->SetValue(wxT("7"));
 		}
+		setSettingsSwitch(L"update_freq_image_incr", m_bitmapButton21, m_textCtrl21);
 	}
 #endif
 	if(getSettingsValue(L"max_file_incr", &t, settings))
@@ -190,6 +257,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl131->SetValue(wxT("100"));
 	}
+	setSettingsSwitch(L"max_file_incr", m_bitmapButton131, m_textCtrl131);
 	if(getSettingsValue(L"min_file_incr", &t, settings))
 	{
 		m_textCtrl13->SetValue(wxString(convert(watoi(t)).c_str()));
@@ -198,6 +266,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl13->SetValue(wxT("40"));
 	}
+	setSettingsSwitch(L"min_file_incr", m_bitmapButton13, m_textCtrl13);
 	if(getSettingsValue(L"max_file_full", &t, settings))
 	{
 		m_textCtrl133->SetValue(wxString(convert(watoi(t)).c_str()));
@@ -206,6 +275,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl133->SetValue(wxT("10"));
 	}
+	setSettingsSwitch(L"max_file_full", m_bitmapButton133, m_textCtrl133);
 	if(getSettingsValue(L"min_file_full", &t,settings))
 	{
 		m_textCtrl132->SetValue(wxString(convert(watoi(t)).c_str()));
@@ -214,6 +284,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl132->SetValue(wxT("2"));
 	}
+	setSettingsSwitch(L"min_file_full", m_bitmapButton132, m_textCtrl132);
 #ifdef _WIN32
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
 	{
@@ -225,6 +296,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 		{
 			m_textCtrl134->SetValue(wxT("4"));
 		}
+		setSettingsSwitch(L"min_image_incr", m_bitmapButton134, m_textCtrl134);
 		if(getSettingsValue(L"max_image_incr", &t, settings))
 		{
 			m_textCtrl135->SetValue(wxString(convert(watoi(t)).c_str()));
@@ -233,6 +305,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 		{
 			m_textCtrl135->SetValue(wxT("30"));
 		}
+		setSettingsSwitch(L"max_image_incr", m_bitmapButton135, m_textCtrl135);
 		if(getSettingsValue(L"min_image_full", &t, settings))
 		{
 			m_textCtrl136->SetValue(wxString(convert(watoi(t)).c_str()));
@@ -241,6 +314,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 		{
 			m_textCtrl136->SetValue(wxT("2"));
 		}
+		setSettingsSwitch(L"min_image_full", m_bitmapButton136, m_textCtrl136);
 		if(getSettingsValue(L"max_image_full", &t, settings))
 		{
 			m_textCtrl137->SetValue(wxString(convert(watoi(t)).c_str()));
@@ -249,6 +323,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 		{
 			m_textCtrl137->SetValue(wxT("5"));
 		}
+		setSettingsSwitch(L"max_image_full", m_bitmapButton137, m_textCtrl137);
 	}
 #endif
 	if(settings->getValue(L"computername", &t) )
@@ -274,6 +349,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 			m_textCtrl17->SetValue(wxT("1-7/0-24"));
 		}
 	}
+	setSettingsSwitch(L"backup_window_incr_file", m_bitmapButton17, m_textCtrl17);
 	if(getSettingsValue(L"exclude_files", &t, settings))
 	{
 		m_textCtrl16->SetValue(t);
@@ -282,6 +358,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl16->SetValue(wxT(""));
 	}
+	setSettingsSwitch(L"exclude_files", m_bitmapButton16, m_textCtrl16);
 	if(getSettingsValue(L"include_files", &t, settings))
 	{
 		m_textCtrl161->SetValue(t);
@@ -290,6 +367,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl161->SetValue(wxT(""));
 	}
+	setSettingsSwitch(L"include_files", m_bitmapButton161, m_textCtrl161);
 	if(getSettingsValue(L"startup_backup_delay", &t, settings))
 	{
 		m_textCtrl19->SetValue(convert(watoi(t)/60));
@@ -298,6 +376,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrl19->SetValue(wxT("0"));
 	}
+	setSettingsSwitch(L"startup_backup_delay", m_bitmapButton19, m_textCtrl19);
 #ifdef _WIN32
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
 	{
@@ -309,6 +388,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 		{
 			m_textCtrl23->SetValue(wxT("C"));
 		}
+		setSettingsSwitch(L"image_letters", m_bitmapButton23, m_textCtrl23);
 	}
 #endif
 	if(getSettingsValue(L"internet_mode_enabled", &t, settings) && t==L"true")
@@ -327,6 +407,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_checkBoxInternetFullFile->SetValue(false);
 	}
+	setSettingsSwitch(L"internet_full_file_backups", m_bitmapButtonInternetFullFile, m_checkBoxInternetFullFile);
 #ifdef _WIN32
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
 	{
@@ -338,24 +419,30 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 		{
 			m_checkBoxInternetImage->SetValue(false);
 		}
+		setSettingsSwitch(L"internet_image_backups", m_bitmapButtonInternetImage, m_checkBoxInternetImage);
 	}
 #endif
-	if(getSettingsValue(L"internet_server", &t, settings))
+	std::wstring internet_server_url;
+	getSettingsValue(L"internet_server", &internet_server_url, settings);
+	std::wstring internet_server_port;
+	getSettingsValue(L"internet_server_port", &internet_server_port, settings);
+
+	if (internet_server_url.find(L"urbackup://") != 0
+		&& internet_server_url.find(L"wss://") != 0
+		&& internet_server_url.find(L"ws://") != 0)
 	{
-		m_textCtrlInternetServer->SetValue(t);
-	}
-	else
-	{
-		m_textCtrlInternetServer->SetValue(wxT(""));
-	}
-	if(getSettingsValue(L"internet_server_port", &t, settings))
-	{
-		m_textCtrlInternetServerPort->SetValue(t);
-	}
-	else
-	{
-		m_textCtrlInternetServerPort->SetValue(wxT("55415"));
+		if (watoi(internet_server_port) == 55415)
+		{
+			internet_server_url = "urbackup://" + internet_server_url;
+		}
+		else
+		{
+			internet_server_url = "urbackup://" + internet_server_url + ":" + internet_server_port;
+		}
 	}
+
+	m_textCtrlInternetServer->SetValue(internet_server_url);
+
 	if (getSettingsValue(L"internet_server_proxy", &t, settings))
 	{
 		m_textCtrlInternetServerProxy->SetValue(t);
@@ -383,6 +470,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrlLocalSpeed->SetValue(wxT("-"));
 	}
+	setSettingsSwitch(L"local_speed", m_bitmapButtonLocalSpeed, m_textCtrlLocalSpeed);
 	if(getSettingsValue(L"internet_speed", &t, settings))
 	{
 		if(watoi(t)>0) 
@@ -394,6 +482,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_textCtrlInternetSpeed->SetValue(wxT("-"));
 	}
+	setSettingsSwitch(L"internet_speed", m_bitmapButtonInternetSpeed, m_textCtrlInternetSpeed);
 	if(getSettingsValue(L"internet_compress", &t, settings) && t==L"false")
 	{
 		m_checkBoxInternetCompress->SetValue(false);
@@ -402,6 +491,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_checkBoxInternetCompress->SetValue(true);
 	}
+	setSettingsSwitch(L"internet_compress", m_bitmapButtonInternetCompress, m_checkBoxInternetCompress);
 	if(getSettingsValue(L"internet_encrypt", &t, settings) && t==L"false")
 	{
 		m_checkBoxInternetEncrypt->SetValue(false);
@@ -410,6 +500,7 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	{
 		m_checkBoxInternetEncrypt->SetValue(true);
 	}
+	setSettingsSwitch(L"internet_encrypt", m_bitmapButtonInternetEncrypt, m_checkBoxInternetEncrypt);
 
 	m_textCtrlInternetSpeed->SetValidator(getDigitSlashValidator());
 	m_textCtrlLocalSpeed->SetValidator(getDigitSlashValidator());	
@@ -431,9 +522,9 @@ Settings::Settings(wxWindow* parent) : GUISettings(parent)
 	}
 #endif
 	m_textCtrl19->SetValidator(wxTextValidator(wxFILTER_DIGITS));
-	m_textCtrlInternetServerPort->SetValidator(wxTextValidator(wxFILTER_DIGITS));
 	m_textCtrl15->SetValidator(getPathValidator());
 	Show(true);
+	init_complete = true;
 }
 
 Settings::~Settings(void)
@@ -485,7 +576,31 @@ void Settings::OnOkClick( wxCommandEvent& event )
 #endif
 	bool internet_mode_enabled=m_checkBoxInternetEnabled->GetValue();
 	wxString internet_server=m_textCtrlInternetServer->GetValue();
-	wxString internet_server_port=m_textCtrlInternetServerPort->GetValue();
+	wxString internet_server_port = L"55415";
+
+	if (!internet_server.empty() &&
+		internet_server.find("urbackup://") != 0 &&
+		internet_server.find("wss://") != 0 &&
+		internet_server.find("ws://") != 0)
+	{
+		internet_server = "urbackup://" + internet_server;
+	}
+
+	if (internet_server.find("urbackup://") == 0)
+	{
+		wxString hostname = internet_server.substr(11);
+		if (hostname.find(":") != std::string::npos)
+		{
+			internet_server_port = getafter(L":", hostname.ToStdWstring());
+			internet_server = getuntil(L":", hostname.ToStdWstring());
+		}
+		else
+		{
+			internet_server_port = L"55415";
+			internet_server = hostname;
+		}
+	}
+
 	wxString internet_server_proxy = m_textCtrlInternetServerProxy->GetValue();
 #ifdef _WIN32
 	bool internet_image_backups=false;
@@ -512,13 +627,15 @@ void Settings::OnOkClick( wxCommandEvent& event )
 	long l_internet_server_port;
 	long l_internet_speed, l_local_speed;
 
-	if(update_freq_incr.ToCDouble(&l_update_freq_incr)==false )
+	if(!update_freq_incr.ToCDouble(&l_update_freq_incr)
+		&& settings_info[L"update_freq_incr"].use==c_use_value_client )
 	{
 		wxMessageBox( _("The incremental backup interval is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 		m_textCtrl1->SetFocus();
 		return;
 	}
-	if(update_freq_full.ToLong(&l_update_freq_full)==false )
+	if(!update_freq_full.ToLong(&l_update_freq_full)
+		&& settings_info[L"update_freq_full"].use==c_use_value_client)
 	{
 		wxMessageBox( _("The full backup interval  is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 		m_textCtrl2->SetFocus();
@@ -527,13 +644,16 @@ void Settings::OnOkClick( wxCommandEvent& event )
 #ifdef _WIN32
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
 	{
-		if(update_freq_image_full.ToLong(&l_update_freq_image_full)==false && m_checkBox1->GetValue() )
+		if(!update_freq_image_full.ToLong(&l_update_freq_image_full) 
+			&& m_checkBox1->GetValue()
+			&& settings_info[L"update_freq_image_full"].use == c_use_value_client)
 		{
 			wxMessageBox( _("The full image backup interval  is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 			m_textCtrl22->SetFocus();
 			return;
 		}
-		if(update_freq_image_incr.ToLong(&l_update_freq_image_incr)==false && m_checkBox1->GetValue()  )
+		if(!update_freq_image_incr.ToLong(&l_update_freq_image_incr) && m_checkBox1->GetValue()
+			&& settings_info[L"update_freq_image_incr"].use == c_use_value_client)
 		{
 			wxMessageBox( _("The incremental image backup time is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 			m_textCtrl21->SetFocus();
@@ -541,25 +661,29 @@ void Settings::OnOkClick( wxCommandEvent& event )
 		}
 	}
 #endif
-	if(max_file_incr.ToLong(&l_max_file_incr)==false )
+	if(!max_file_incr.ToLong(&l_max_file_incr)
+		&& settings_info[L"max_file_incr"].use == c_use_value_client)
 	{
 		wxMessageBox( _("The maximal number of incremental file backups  is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 		m_textCtrl131->SetFocus();
 		return;
 	}
-	if(min_file_incr.ToLong(&l_min_file_incr)==false )
+	if(!min_file_incr.ToLong(&l_min_file_incr)
+		&& settings_info[L"min_file_incr"].use == c_use_value_client)
 	{
 		wxMessageBox( _("The minimal number of incremental file backups is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 		m_textCtrl13->SetFocus();
 		return;
 	}
-	if(max_file_full.ToLong(&l_max_file_full)==false )
+	if(!max_file_full.ToLong(&l_max_file_full)
+		&& settings_info[L"max_file_full"].use == c_use_value_client)
 	{
 		wxMessageBox( _("The maximal number of full file backups is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 		m_textCtrl133->SetFocus();
 		return;
 	}
-	if(min_file_full.ToLong(&l_min_file_full)==false )
+	if(!min_file_full.ToLong(&l_min_file_full)
+		&& settings_info[L"min_file_full"].use == c_use_value_client)
 	{
 		wxMessageBox( _("Minimal number of full file backups is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 		m_textCtrl132->SetFocus();
@@ -568,25 +692,29 @@ void Settings::OnOkClick( wxCommandEvent& event )
 #ifdef _WIN32
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
 	{
-		if(min_image_incr.ToLong(&l_min_image_incr)==false )
+		if(!min_image_incr.ToLong(&l_min_image_incr)
+			&& settings_info[L"min_image_incr"].use == c_use_value_client)
 		{
 			wxMessageBox( _("Minimal number of incremental image backups is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 			m_textCtrl134->SetFocus();
 			return;
 		}
-		if(max_image_incr.ToLong(&l_max_image_incr)==false )
+		if(!max_image_incr.ToLong(&l_max_image_incr)
+			&& settings_info[L"max_image_incr"].use == c_use_value_client)
 		{
 			wxMessageBox( _("Maximal number of incremental image backups is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 			m_textCtrl135->SetFocus();
 			return;
 		}
-		if(min_image_full.ToLong(&l_min_image_full)==false )
+		if(!min_image_full.ToLong(&l_min_image_full)
+			&& settings_info[L"min_image_full"].use == c_use_value_client)
 		{
 			wxMessageBox( _("Minimal number of full image backups is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 			m_textCtrl136->SetFocus();
 			return;
 		}
-		if(max_image_full.ToLong(&l_max_image_full)==false )
+		if(!max_image_full.ToLong(&l_max_image_full)
+			&& settings_info[L"max_image_full"].use == c_use_value_client)
 		{
 			wxMessageBox( _("Maximal number of full image backups is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 			m_textCtrl137->SetFocus();
@@ -594,7 +722,8 @@ void Settings::OnOkClick( wxCommandEvent& event )
 		}
 	}
 #endif
-	if(startup_backup_delay.ToLong(&l_startup_backup_delay)==false)
+	if(!startup_backup_delay.ToLong(&l_startup_backup_delay)
+		&& settings_info[L"start_backup_delay"].use == c_use_value_client)
 	{
 		wxMessageBox( _("Backup delay after system start is not a number"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
 		m_textCtrl19->SetFocus();
@@ -604,7 +733,8 @@ void Settings::OnOkClick( wxCommandEvent& event )
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
 	{
 		std::string s_image_letters=image_letters.ToUTF8();
-		if(strlower(s_image_letters)!="all" && strlower(s_image_letters)!="all_nonusb")
+		if(strlower(s_image_letters)!="all" && strlower(s_image_letters)!="all_nonusb"
+			&& settings_info[L"image_letters"].use == c_use_value_client)
 		{
 			std::vector<std::string> img_paths;
 			Tokenize(s_image_letters, img_paths, ";,");
@@ -628,10 +758,10 @@ void Settings::OnOkClick( wxCommandEvent& event )
 		}		
 	}
 #endif
-	if(internet_server_port.ToLong(&l_internet_server_port)==false)
+	if(!internet_server_port.ToLong(&l_internet_server_port))
 	{
 		wxMessageBox( _("The server port is not valid"), wxT("UrBackup"), wxOK | wxCENTRE | wxICON_ERROR);
-		m_textCtrlInternetServerPort->SetFocus();
+		m_textCtrlInternetServer->SetFocus();
 		return;
 	}
 
@@ -640,10 +770,10 @@ void Settings::OnOkClick( wxCommandEvent& event )
 	local_speed.ToLong(&l_local_speed);
 	l_local_speed*=(1024*1024)/8;
 
-	l_update_freq_image_full_orig=l_update_freq_image_full;
 #ifdef _WIN32
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
 	{
+		l_update_freq_image_full_orig = l_update_freq_image_full;
 		if(m_checkBox1->GetValue()==false)
 		{
 			if(l_update_freq_image_full>0)
@@ -654,7 +784,55 @@ void Settings::OnOkClick( wxCommandEvent& event )
 	}
 #endif
 
-	std::map<std::string, std::string> n_vals;
+	std::string s_data;
+
+	s_data += std::string("internet_mode_enabled=")+nconvert(internet_mode_enabled)+"\n";
+	s_data += std::string("internet_server=")+ internet_server.ToUTF8() + "\n";
+	s_data += std::string("internet_server_port=") + nconvert(l_internet_server_port) + "\n";
+	s_data += std::string("internet_server_proxy=") + internet_server_proxy.ToUTF8() + "\n";
+	s_data += std::string("internet_authkey=") + internet_authkey.ToUTF8() + "\n";
+	s_data += std::string("computername=") + computername.ToUTF8() + "\n";
+
+	int64 ctime = wxGetUTCTimeMillis().GetValue() / 1000;
+
+	for (std::map<std::wstring, SSetting>::iterator it = settings_info.begin(); it != settings_info.end(); ++it)
+	{
+		std::wstring old_val;
+		std::wstring old_use;
+		if (it->second.use != c_use_group
+			&& it->second.use!=c_use_value
+			&& ((!settings->getValue(it->first+L".client", &old_val) &&
+				old_val!=it->second.value_client) ||
+				( !settings->getValue(it->first + L".use", &old_use) &&
+				old_use!=convert(it->second.use) ) )  )
+		{
+			std::string key(wxString(it->first).ToUTF8());
+			s_data += key + ".client=" + wxString(it->second.value_client).ToUTF8() + "\n";
+			s_data += key + ".use=" + nconvert(it->second.use) + "\n";
+			int64 use_lm = 0;
+			std::wstring old_use_lm_str;
+			if (!settings->getValue(it->first + L".use_lm", &old_use_lm_str)
+				|| old_use_lm_str.empty())
+				use_lm = ctime;
+			else
+				wxString(old_use_lm_str).ToLongLong(&use_lm);
+
+
+			if (convert(it->second.use) != old_use)
+			{
+				if (use_lm > ctime)
+					++use_lm;
+				else
+					use_lm = ctime;
+			}
+
+			s_data += key + ".use_lm=" + nconvert(use_lm) + "\n";
+		}
+	}
+
+	Connector::updateSettings(s_data);
+
+	/*std::map<std::string, std::string> n_vals;
 	n_vals["update_freq_incr"]=nconvert(static_cast<float>(l_update_freq_incr*60.f*60.f));
 	n_vals["update_freq_full"]=nconvert(l_update_freq_full*24*60*60);
 	if(!MyTimer::hasCapability(DONT_DO_IMAGE_BACKUPS, capa))
@@ -711,7 +889,7 @@ void Settings::OnOkClick( wxCommandEvent& event )
 	n_vals["internet_encrypt"]=nconvert(internet_encrypt);
 	n_vals["internet_compress"]=nconvert(internet_compress);
 
-	Connector::updateSettings(mergeNewSettings(settings, n_vals));
+	Connector::updateSettings(mergeNewSettings(settings, n_vals));*/
 
 	Close();
 }
@@ -735,47 +913,187 @@ void Settings::OnDisableImageBackups( wxCommandEvent& event )
 	}		
 }
 
-std::string Settings::mergeNewSettings(CFileSettingsReader * settings, const std::map<std::string, std::string>& n_vals)
+void Settings::OnBitmapBtnClick(wxCommandEvent & event)
 {
-	std::string ndata;
+	std::wstring& key = button_ids[event.GetId()];
+	SSetting& setting = settings_info[key];
 
-	std::vector<std::wstring> keys = settings->getKeys();
+	int use = setting.use;
 
-	for (std::map<std::string, std::string>::const_iterator it = n_vals.begin(); it != n_vals.end(); ++it)
+	if (use == c_use_group)
+		use = c_use_value;
+	else if (use == c_use_value)
+		use = c_use_value_client;
+	else
+		use = c_use_group;
+
+	std::wstring val;
+	if (use == c_use_value_client)
 	{
-		const std::string &nkey = it->first;
-		std::string def_value;
-		bool found_key = false;
-
-		std::wstring key_w = ConvertToUnicode(nkey);
-		for (size_t i = 0; i<keys.size(); ++i)
-		{
-			if (keys[i] == key_w)
-			{
-				found_key = true;
-				break;
-			}
-		}
-
-		if (found_key || !settings->getValue(it->first + "_def", &def_value) || def_value != it->second)
-		{
-			ndata += nkey + "=" + it->second + "\n";
-		}
+		setting.btn->SetBitmapLabel(fa_client_img_scaled);
+		val = setting.value_client;
+	}
+	else if (use == c_use_group)
+	{
+		setting.btn->SetBitmapLabel(fa_lock_img_scaled);
+		val = setting.value_group;
+	}
+	else
+	{
+		setting.btn->SetBitmapLabel(fa_home_img_scaled);
+		val = setting.value_home;
 	}
 
-	for (size_t i = 0; i<keys.size(); ++i)
+
+	setting.use = use;
+
+	wxCheckBox* checkBox = dynamic_cast<wxCheckBox*>(setting.ctrl);
+	if (checkBox != NULL)
 	{
-		std::string key = ConvertToUTF8(keys[i]);
-		std::map<std::string, std::string>::const_iterator iter = n_vals.find(key);
-		if (iter == n_vals.end())
-		{
-			std::wstring val;
-			if (settings->getValue(keys[i], &val))
-			{
-				ndata += key + "=" + ConvertToUTF8(val) + "\n";
-			}
-		}
+		checkBox->SetValue(val == L"1" || val == L"true");
 	}
 
-	return ndata;
+	wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>(setting.ctrl);
+	if (textCtrl != NULL)
+	{
+		textCtrl->ChangeValue(transformValToUI(key, val));
+	}
+}
+
+void Settings::OnCtlChange(wxCommandEvent & event)
+{
+	if (!init_complete)
+		return;
+
+	std::wstring key = ctrl_ids[event.GetId()];
+	
+	SSetting& setting = settings_info[key];
+
+	setting.use = c_use_value_client;
+
+	wxCheckBox* checkBox = dynamic_cast<wxCheckBox*>(setting.ctrl);
+	if (checkBox != NULL)
+	{
+		setting.value_client = convert(checkBox->GetValue());
+	}
+
+	wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>(setting.ctrl);
+	if (textCtrl != NULL)
+	{
+		setting.value_client = transformValFromUI(key, textCtrl->GetValue().ToStdWstring());
+	}	
+
+	setting.btn->SetBitmapLabel(fa_client_img_scaled);
+}
+
+std::wstring Settings::transformValToUI(const std::wstring & key, const std::wstring & val)
+{
+	if (key == "update_freq_incr")
+	{
+		return convert(watoi(val) / 60.f / 60.f);
+	}
+	else if (key == "update_freq_full")
+	{
+		return convert(watoi(val) / 24 / 60 / 60);
+	}
+	else if (key == "internet_speed")
+	{
+		return convert(watoi(val) / (1024 / 8));
+	}
+	else if (key == "local_speed")
+	{
+		return convert(watoi(val) / ((1024*1024) / 8));
+	}
+	else if (key == "update_freq_image_full"
+		|| key=="update_freq_image_incr")
+	{
+		int ret = watoi(val) * 24 * 60 * 60;
+		if (!m_checkBox1->GetValue()
+			&& ret>0)
+		{
+			ret *= -1;
+		}
+		return convert(ret);
+	}
+	else if (key == "startup_backup_delay")
+	{
+		return convert(watoi(val) / 60);
+	}
+
+	return val;
+}
+
+std::wstring Settings::transformValFromUI(const std::wstring & key, const std::wstring & val)
+{
+	if (key == "update_freq_incr")
+	{
+		return convert(watoi(val) * 60.f * 60.f);
+	}
+	else if (key == "update_freq_full")
+	{
+		return convert(watoi(val)* 24 * 60 * 60);
+	}
+	else if (key == "internet_speed")
+	{
+		return convert(watoi(val) * (1024 / 8) );
+	}
+	else if (key == "local_speed")
+	{
+		return convert(watoi(val) * ((1024*1024) / 8));
+	}
+	else if (key == "update_freq_image_full"
+		|| key=="update_freq_image_incr")
+	{
+		int ret = watoi(val) / 24 / 60 / 60;
+		m_checkBox1->SetValue(ret > 0);
+		return convert(ret);
+	}
+	else if (key == "startup_backup_delay")
+	{
+		return convert(watoi(val) * 60);
+	}
+
+	return val;
+}
+
+void Settings::setSettingsSwitch(const std::wstring & key, wxBitmapButton* btn, wxWindow* ctrl)
+{
+	std::wstring use_str;
+	settings->getValue(key + L".use", &use_str);
+
+	int use = watoi(use_str);;
+
+	if (use == c_use_value_client)
+	{
+		btn->SetBitmapLabel(fa_client_img_scaled);
+	}
+	else if (use == c_use_group)
+	{
+		btn->SetBitmapLabel(fa_lock_img_scaled);
+	}
+	else
+	{
+		btn->SetBitmapLabel(fa_home_img_scaled);
+	}
+
+	ctrl_ids[ctrl->GetId()] = key;
+	button_ids[btn->GetId()] = key;
+	SSetting& setting = settings_info[key];
+	setting.btn = btn;
+	setting.ctrl = ctrl;
+	setting.use = use;
+	if (!settings->getValue(key + L".client", &setting.value_client))
+	{
+		settings->getValue(key, &setting.value_client);
+	}
+	if (!settings->getValue(key + L".home", &setting.value_home))
+	{
+		if(!settings->getValue(key+L".def", &setting.value_client))
+			settings->getValue(key, &setting.value_client);
+	}
+	if (!settings->getValue(key + L".group", &setting.value_group))
+	{
+		if (!settings->getValue(key + L"_def", &setting.value_client))
+			settings->getValue(key, &setting.value_client);
+	}
 }
