@@ -142,8 +142,8 @@ HRESULT ModifyPrivilege(
 
 #ifdef __APPLE__
 extern "C" void bring_to_foreground();
-extern "C" void register_login_item();
 extern "C" void remove_login_item();
+extern "C" void check_full_disk_access();
 #endif
 
 class TheFrame : public wxFrame {
@@ -290,6 +290,12 @@ bool MyApp::OnInit()
 	//SetCurrentDirectoryW(ExtractFilePath(buf).c_str() );
 	g_res_path=ConvertToUTF8(ExtractFilePath(buf))+"\\";
 	res_path=wxString(g_res_path.c_str(), wxMBConvUTF8());
+#elif defined(__APPLE__)
+	wxFileName base(wxStandardPaths::Get().GetExecutablePath());
+	base.AppendDir(wxT("share"));
+	base.AppendDir(wxT("urbackup"));
+	res_path=base.GetPathWithSep();
+	g_res_path=res_path;
 #else
 	res_path=wxT(DATADIR "/urbackup/");
 	g_res_path= DATADIR "/urbackup/";
@@ -351,7 +357,6 @@ bool MyApp::OnInit()
 	case wxLANGUAGE_SPANISH_GUATEMALA:
 	case wxLANGUAGE_SPANISH_HONDURAS:
 	case wxLANGUAGE_SPANISH_MEXICAN:
-	case wxLANGUAGE_SPANISH_MODERN:
 	case wxLANGUAGE_SPANISH_NICARAGUA:
 	case wxLANGUAGE_SPANISH_PANAMA:
 	case wxLANGUAGE_SPANISH_PARAGUAY:
@@ -372,7 +377,6 @@ bool MyApp::OnInit()
 		g_lang="zh_CN";
 		break;
 	case wxLANGUAGE_CHINESE_TRADITIONAL:
-	case wxLANGUAGE_CHINESE_TAIWAN:
 		lang= sysdef;
 		g_lang="zh_TW";
 		break;
@@ -507,6 +511,10 @@ bool MyApp::OnInit()
 		timer=new MyTimer;
 
 		timer->Start(1000);
+        
+        #ifdef __APPLE__
+        check_full_disk_access();
+        #endif
 	}
 	else if(cmd==wxT("settings"))
 	{
@@ -595,11 +603,6 @@ bool MyApp::OnInit()
 		wxExecute("/bin/sh \"" + uninstaller + "\"", wxEXEC_SYNC, NULL, NULL);
 		wxExit();
 	}
-	else if(cmd==wxT("register_login_item"))
-	{
-		register_login_item();
-		exit(0);
-	}
 	else if(cmd==wxT("remove_login_item"))
 	{
 		remove_login_item();
@@ -865,6 +868,8 @@ void MyTimer::Notify()
 		}
 	}
 
+	/* Users might accidentally click on it. Disable for now.
+	
 	if(!status.new_server.empty() && tray)
 	{
 #ifndef wxUSE_TASKBARICON_BALLOONS
@@ -874,7 +879,7 @@ void MyTimer::Notify()
 			tray->BalloonActionNewServer(status.new_server);
 			tray->ShowBalloon(_("UrBackup: New server"), _("A new backup server has been found. Click here to use it"), 180000, wxICON_INFORMATION);
 #endif
-	}
+	}*/
 
 	if(status.ask_restore_ok>ask_restore_ok)
 	{
@@ -959,6 +964,7 @@ void MyTimer::resetDisplayedUpdateInfo(void)
 #ifndef DD_RELEASE
 int main(int argc, char *argv[])
 {
+	wxDISABLE_DEBUG_SUPPORT();
 	#ifdef __APPLE__
 	if(argc>1 && wxString(argv[1])==wxT("daemon"))
 	{
@@ -993,4 +999,4 @@ int main(int argc, char *argv[])
 }
 #else
 IMPLEMENT_APP(MyApp)
-#endif
\ No newline at end of file
+#endif
