@@ -37,7 +37,7 @@
 #define PWFILE_CHANGE VARDIR "/urbackup/pw_change.txt"
 #else
 #include <Windows.h>
-#define PACKAGE_VERSION "2.4.10.0"
+#define PACKAGE_VERSION "2.5.22.0"
 #define VARDIR ""
 #define PWFILE "pw.txt"
 #define PWFILE_CHANGE "pw_change.txt"
@@ -900,6 +900,25 @@ int action_set_settings(std::vector<std::string> args)
 		"New value to set the setting to",
 		false, "setting value", cmd);
 
+	TCLAP::SwitchArg no_merge_arg("n", "no-merge",
+		"Don't merge server and client settings if possible", cmd);
+
+	TCLAP::ValueArg<std::string> server_url_arg("", "server-url",
+		"URL of server to connect to",
+		false, "", "url", cmd);
+
+	TCLAP::ValueArg<std::string> name_arg("", "name",
+		"Client name",
+		false, "", "string", cmd);
+
+	TCLAP::ValueArg<std::string> authkey_arg("", "authkey",
+		"Server authentication key for client",
+		false, "", "string", cmd);
+
+	TCLAP::ValueArg<std::string> proxy_arg("", "proxy",
+		"HTTP CONNECT proxy to use to connect to server",
+		false, "", "url", cmd);
+
 	cmd.parse(args);
 
 	if (key_arg.getValue().size() != value_arg.getValue().size())
@@ -913,13 +932,88 @@ int action_set_settings(std::vector<std::string> args)
 		return 3;
 	}
 
+	str_map arg_settings;
+
+	if (server_url_arg.isSet())
+	{
+		std::vector<std::string> server_urls;
+		Tokenize(server_url_arg.getValue(), server_urls, ";");
+		std::string internet_server;
+		std::string internet_server_port;
+		for (size_t i = 0; i < server_urls.size(); ++i)
+		{
+			std::string server_url = server_urls[i];
+			std::string server_port = "55415";
+
+			if (server_url.find("urbackup://") != 0 &&
+				server_url.find("wss://") != 0 &&
+				server_url.find("ws://") != 0)
+			{
+				std::cerr << "Server URL must start with urbackup://, wss:// or ws://" << std::endl;
+				return 4;
+			}
+
+			if (server_url.find("urbackup://") == 0)
+			{
+				std::string hostname = server_url.substr(11);
+				if (hostname.find(":") != std::string::npos)
+				{
+					server_port = getafter(":", server_url);
+				}
+				server_url = hostname;
+			}
+
+			if (!internet_server.empty())
+				internet_server += ";";
+			if (!internet_server_port.empty())
+				internet_server_port += ";";
+
+			internet_server += server_url;
+			internet_server_port += server_port;
+		}
+
+		arg_settings["internet_server_port"] = internet_server_port;
+		arg_settings["internet_server"] = internet_server;
+		arg_settings["internet_mode_enabled"] = "true";
+	}
+
+	if (authkey_arg.isSet())
+	{
+		arg_settings["internet_authkey"] = authkey_arg.getValue();
+		arg_settings["internet_mode_enabled"] = "true";
+	}
+
+	if (name_arg.isSet())
+	{
+		arg_settings["computername"] = name_arg.getValue();
+	}
+
+	if (proxy_arg.isSet())
+	{
+		arg_settings["internet_server_proxy"] = proxy_arg.getValue();
+		arg_settings["internet_mode_enabled"] = "true";
+	}
+
 	std::string s_settings;
 	for (size_t i = 0; i < key_arg.getValue().size(); ++i)
 	{
-		s_settings += key_arg.getValue()[i] + "=" + value_arg.getValue()[i] + "\n";
+		std::string key = key_arg.getValue()[i];
+		if(arg_settings.find(key)==arg_settings.end())
+			s_settings += key + "=" + value_arg.getValue()[i] + "\n";
 	}
 
-	s_settings += "keep_old_settings=true\n";
+	for (str_map::const_iterator it = arg_settings.begin();
+		it != arg_settings.end(); ++it)
+	{
+		s_settings += it->first + "=" + it->second + "\n";
+	}
+
+	s_settings += "set_client_settings=1\n";
+
+	if (!no_merge_arg.getValue())
+	{
+		s_settings += "merge_client_settings=0\n";
+	}
 
 	bool no_perm;
 	bool b = Connector::updateSettings(s_settings, no_perm);
@@ -1030,7 +1124,7 @@ std::string getDefaultDirname(const std::vector<SBackupDir>& dirs, const std::st
 	std::string dirname = removeChars(ExtractFileName(path));
 
 	if (dirname.empty())
-		dirname = "root";
+		dirname = "rootfs";
 
 	if (findPathName(dirs, dirname))
 	{
@@ -1158,6 +1252,17 @@ int action_add_backupdir(std::vector<std::string> args)
 
 	SBackupDir new_dir;
 	new_dir.path = path_arg.getValue();
+
+	if (!os_path_absolute(new_dir.path))
+	{
+		new_dir.path = os_get_final_path(new_dir.path);
+	}
+
+	if (!new_dir.path.empty() && new_dir.path[new_dir.path.size() - 1] == os_file_sep()[0])
+	{
+		new_dir.path.erase(new_dir.path.size() - 1, 1);
+	}
+
 	if (name_arg.getValue().empty())
 	{
 		new_dir.name = getDefaultDirname(backup_dirs, new_dir.path);
@@ -1293,6 +1398,7 @@ int action_list_backupdirs(std::vector<std::string> args)
 
 	bool has_virtual_client = false;
 	bool has_group = false;
+	bool has_server_default = false;
 
 	for (size_t i = 0; i < backup_dirs.size(); ++i)
 	{
@@ -1304,6 +1410,8 @@ int action_list_backupdirs(std::vector<std::string> args)
 		{
 			has_group = true;
 		}
+		if (backup_dirs[i].server_default)
+			has_server_default = true;
 	}
 
 	std::vector<std::vector<std::string> > tab;
@@ -1320,6 +1428,10 @@ int action_list_backupdirs(std::vector<std::string> args)
 		tab_header.push_back("VIRTUAL CLIENT");
 	}
 	tab_header.push_back("FLAGS");
+	if (has_server_default)
+	{
+		tab_header.push_back("CONFIGURED ON SERVER");
+	}
 
 	tab.push_back(tab_header);
 
@@ -1354,6 +1466,18 @@ int action_list_backupdirs(std::vector<std::string> args)
 
 		row.push_back(backup_dirs[i].flags);
 
+		if (has_server_default)
+		{
+			if (backup_dirs[i].server_default)
+			{
+				row.push_back("Yes");
+			}
+			else
+			{
+				row.push_back("No");
+			}
+		}
+
 		tab.push_back(row);
 	}
 
@@ -1394,20 +1518,35 @@ int action_remove_backupdir(std::vector<std::string> args)
 	}
 
 	bool del_ok = false;
+	bool del_server_default = true;
 
 	for (size_t i = 0; i < backup_dirs.size();)
 	{
 		if (!name_arg.getValue().empty()
 			&& backup_dirs[i].name == name_arg.getValue())
 		{
-			backup_dirs.erase(backup_dirs.begin() + i);
-			del_ok = true;
+			if (backup_dirs[i].server_default)
+			{
+				del_server_default = true;
+			}
+			else
+			{
+				backup_dirs.erase(backup_dirs.begin() + i);
+				del_ok = true;
+			}
 		}
 		else if (!path_arg.getValue().empty()
 			&& backup_dirs[i].path == path_arg.getValue())
 		{
-			backup_dirs.erase(backup_dirs.begin() + i);
-			del_ok = true;
+			if (backup_dirs[i].server_default)
+			{
+				del_server_default = true;
+			}
+			else
+			{
+				backup_dirs.erase(backup_dirs.begin() + i);
+				del_ok = true;
+			}
 		}
 		else
 		{
@@ -1417,7 +1556,14 @@ int action_remove_backupdir(std::vector<std::string> args)
 
 	if (!del_ok)
 	{
-		std::cerr << "Backup directory to remove not found" << std::endl;
+		if (del_server_default)
+		{
+			std::cerr << "Backup directory was configured on the server. Please remove it there" << std::endl;
+		}
+		else
+		{
+			std::cerr << "Backup directory to remove not found" << std::endl;
+		}
 		return 1;
 	}
 
