@@ -42,6 +42,23 @@
 #include <memory>
 #include "../cryptoplugin/ICryptoFactory.h"
 #include "../fsimageplugin/IFSImageFactory.h"
+#include "../urbackupcommon/json.h"
+#ifndef _WIN32
+#include "../config.h"
+#include <sys/ioctl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
+#ifdef HAVE_LINUX_FS_H
+#include <linux/fs.h>
+#endif
+
+#ifndef BLKRRPART
+#define BLKRRPART  _IO(0x12,95)
+#endif
+
+#endif
 
 #ifdef _WIN32
 const std::string pw_file="pw.txt";
@@ -59,6 +76,26 @@ extern IFSImageFactory* image_fak;
 
 namespace
 {
+	bool rereadPartitionLayout(const std::string& devfn)
+	{
+		bool ret = true;
+#ifndef _WIN32
+		int fd = open(devfn.c_str(), O_NONBLOCK | O_RDWR);
+		if (fd == -1)
+			return false;
+
+		int rc = ioctl(fd, BLKRRPART, NULL);
+
+		if (rc == -1)
+		{
+			ret = false;
+			Server->Log("Error re-reading partition layout. " + os_last_error_str(), LL_WARNING);
+		}
+
+		close(fd);
+#endif
+		return ret;
+	}
 
 struct LoginData
 {
@@ -67,6 +104,7 @@ struct LoginData
 	bool has_login_data;
 	std::string username;
 	std::string password;
+	std::string token;
 };
 
 std::string trim2(const std::string &str)
@@ -249,8 +287,10 @@ std::vector<SPasswordSalt> getSalt(const std::string& username, int tries, int *
 			else
 			{
 				Server->Log("SALT error: "+toks[i], LL_ERROR);
+				Server->Log("Username not found on server", LL_ERROR);
 				SPasswordSalt new_salts = {};
 				ret.push_back(new_salts);
+				tries = 0;
 			}
 		}
 	}
@@ -355,6 +395,8 @@ struct SImage
 	std::string letter;
 };
 
+std::vector<SImage> parse_backup_images_output(const std::string& data);
+
 std::vector<SImage> getBackupimages(std::string clientname, int *ec)
 {
 	std::string pw=getFile(pw_file);
@@ -386,34 +428,7 @@ std::vector<SImage> getBackupimages(std::string clientname, int *ec)
 		}
 		else
 		{
-			std::vector<std::string> toks;
-			std::string t=r.substr(1);
-			Tokenize(t, toks, "\n");
-			for(size_t i=0;i<toks.size();++i)
-			{
-				std::vector<std::string> t2;
-				Tokenize(toks[i], t2, "|");
-				if(t2.size()==3 || (t2.size()==4 && t2[0]!="#") )
-				{
-					SImage si;
-					si.id=atoi(t2[0].c_str());
-					si.time_s=os_atoi64(t2[1]);
-					si.time_str=t2[2];
-					if(t2.size()==4)
-					{
-						si.letter=t2[3];
-					}
-					ret.push_back(si);
-				}
-				else if(t2.size()==4 && t2[0]=="#" && !ret.empty())
-				{
-					SImage si;
-					si.id=atoi(t2[1].c_str());
-					si.time_s=os_atoi64(t2[2]);
-					si.time_str=t2[3];
-					ret[ret.size()-1].assoc.push_back(si);
-				}
-			}
+			ret = parse_backup_images_output(r.substr(1));
 		}
 	}
 	Server->destroy(c);
@@ -572,7 +587,18 @@ EDownloadResult downloadImage(int img_id, std::string img_time, std::string outf
 		s_offset="&offset="+convert(dl_status.offset)+"&received_bytes="+convert(dl_status.received);
 	}
 
-	tcpstack.Send(client_pipe.get(), "DOWNLOAD IMAGE#pw="+pw+"&img_id="+convert(img_id)+"&time="+img_time+"&mbr="+convert(mbr)+s_offset);
+	std::string dl_args;
+	if (img_id != 0 || !img_time.empty())
+	{
+		dl_args = "&img_id = "+convert(img_id)+" & time = "+img_time;
+	}
+	else if (login_data.has_login_data
+		&& !login_data.token.empty())
+	{
+		dl_args = "&token=" + EscapeParamString(login_data.token);
+	}
+
+	tcpstack.Send(client_pipe.get(), "DOWNLOAD IMAGE#pw="+pw+ dl_args + "&mbr="+convert(mbr)+s_offset);
 
 	std::string restore_out=outfile;
 	Server->Log("Restoring to "+restore_out);
@@ -797,7 +823,7 @@ int downloadFiles(int backupid, std::string backup_time)
 		return 10;
 	}
 
-	tcpstack.Send(client_pipe.get(), "DOWNLOAD IMAGE#pw="+pw+"&backupid="+convert(backupid)+"&time="+backup_time);
+	tcpstack.Send(client_pipe.get(), "DOWNLOAD FILES#pw="+pw+"&backupid="+convert(backupid)+"&time="+backup_time);
 	int64 starttime = Server->getTimeMS();
 
 	while(Server->getTimeMS()-starttime<60000)
@@ -987,6 +1013,75 @@ namespace
 	}
 }
 
+namespace
+{
+	std::vector<SImage> parse_backup_images_output(const std::string& data)
+	{
+		std::vector<SImage> ret;
+		std::vector<std::string> toks;
+		Tokenize(data, toks, "\n");
+		for (size_t i = 0; i < toks.size(); ++i)
+		{
+			std::vector<std::string> t2;
+			Tokenize(toks[i], t2, "|");
+			if (t2.size() == 3 || (t2.size() == 4 && t2[0] != "#"))
+			{
+				SImage si;
+				si.id = atoi(t2[0].c_str());
+				si.time_s = os_atoi64(t2[1]);
+				si.time_str = t2[2];
+				if (t2.size() == 4)
+				{
+					si.letter = t2[3];
+				}
+				ret.push_back(si);
+			}
+			else if (t2.size() == 4 && t2[0] == "#" && !ret.empty())
+			{
+				SImage si;
+				si.id = atoi(t2[1].c_str());
+				si.time_s = os_atoi64(t2[2]);
+				si.time_str = t2[3];
+				ret[ret.size() - 1].assoc.push_back(si);
+			}
+		}
+		return ret;
+	}
+
+	std::string backup_images_output_to_json(const std::string& data)
+	{
+		std::vector<SImage> images = parse_backup_images_output(data);
+
+		JSON::Array ret;
+		for (size_t i = 0; i < images.size(); ++i)
+		{
+			JSON::Object image;
+			image.set("id", images[i].id);
+			image.set("time_s", images[i].time_s);
+			image.set("time_str", images[i].time_str);
+			image.set("letter", images[i].letter);
+
+			if (!images[i].assoc.empty())
+			{
+				JSON::Array assoc;
+				for (size_t j = 0; j < images[i].assoc.size(); ++j)
+				{
+					JSON::Object image_assoc;
+					image_assoc.set("id", images[i].assoc[j].id);
+					image_assoc.set("time_s", images[i].assoc[j].time_s);
+					image_assoc.set("time_str", images[i].assoc[j].time_str);
+					image_assoc.set("letter", images[i].assoc[j].letter);
+					assoc.add(image_assoc);
+				}
+				image.set("assoc", assoc);
+			}
+			ret.add(image);
+		}
+
+		return ret.stringify(false);
+	}
+}
+
 void do_restore(void)
 {
 	std::string cmd=Server->getServerParameter("restore_cmd");
@@ -1057,22 +1152,28 @@ void do_restore(void)
 				Server->Log("Writing GPT table failed. " + os_last_error_str(), LL_ERROR);
 			}
 
-			Server->Log("Writing GPT backup header...");
-			if (dev->Write(mbrdata.backup_gpt_header_pos, mbrdata.backup_gpt_header) != mbrdata.backup_gpt_header.size())
+			if (mbrdata.backup_gpt_header_pos != -1)
 			{
-				Server->Log("Writing GPT backup header failed. " + os_last_error_str(), LL_ERROR);
+				Server->Log("Writing GPT backup header...");
+				if (dev->Write(mbrdata.backup_gpt_header_pos, mbrdata.backup_gpt_header) != mbrdata.backup_gpt_header.size())
+				{
+					Server->Log("Writing GPT backup header failed. " + os_last_error_str(), LL_ERROR);
+				}
 			}
 
-			Server->Log("Writing GPT backup table...");
-			if (dev->Write(mbrdata.backup_gpt_table_pos, mbrdata.backup_gpt_table) != mbrdata.backup_gpt_table.size())
+			if (mbrdata.backup_gpt_table_pos != -1)
 			{
-				Server->Log("Writing backup GPT table failed. " + os_last_error_str(), LL_ERROR);
+				Server->Log("Writing GPT backup table...");
+				if (dev->Write(mbrdata.backup_gpt_table_pos, mbrdata.backup_gpt_table) != mbrdata.backup_gpt_table.size())
+				{
+					Server->Log("Writing backup GPT table failed. " + os_last_error_str(), LL_ERROR);
+				}
 			}
 		}
 
 		if (mbrdata.extra_data_pos != -1)
 		{
-			Server->Log("Writing extra data...");
+			Server->Log("Writing extra data at position "+convert(mbrdata.extra_data_pos)+" size "+convert(mbrdata.extra_data.size())+" ...");
 			if (dev->Write(mbrdata.extra_data_pos, mbrdata.extra_data) != mbrdata.extra_data.size())
 			{
 				Server->Log("Writing extra data failed. " + os_last_error_str(), LL_ERROR);
@@ -1081,6 +1182,11 @@ void do_restore(void)
 
 		Server->destroy(dev);
 
+		if (out_device.find("/dev/") == 0)
+		{
+			rereadPartitionLayout(out_device);
+		}
+
 		delete []buf;
 		exit(0);
 	}
@@ -1115,17 +1221,68 @@ void do_restore(void)
 		delete []buf;
 		exit(0);
 	}
+	else if (cmd == "login")
+	{
+		LoginData login_data;
+		login_data.has_login_data = true;
+		login_data.username = Server->getServerParameter("username");
+		login_data.password = Server->getServerParameter("password");
+
+		char* pw = getenv("LOGIN_PASSWORD");
+		if (pw != NULL)
+		{
+			login_data.password = pw;
+		}
+		if (do_login(login_data))
+		{
+			exit(0);
+		}
+		else
+		{
+			exit(1);
+		}
+	}
+	else if (cmd == "read_mbr")
+	{
+		std::string dev_fn = Server->getServerParameter("device_fn");
+		std::auto_ptr<IFile> dev(Server->openFile(dev_fn, MODE_READ_DEVICE));
+
+		if (dev.get() == NULL)
+		{
+			Server->Log("Error opening device at " + dev_fn+". "+os_last_error_str(), LL_ERROR);
+			exit(5);
+		}
+
+		bool gpt_style;
+		std::vector<IFSImageFactory::SPartition> partitions = image_fak->readPartitions(dev.get(), gpt_style);
+
+		if (partitions.empty())
+		{
+			Server->Log("No partitions found", LL_ERROR);
+			exit(4);
+		}
+
+		for (size_t i = 0; i < partitions.size(); ++i)
+		{
+			std::cout << "partition " << (i+1) << " off " << partitions[i].offset << " size " << partitions[i].length << std::endl;
+		}
+
+		exit(0);
+	}
 	else if(cmd=="help")
 	{
 		Server->Log("restore_cmd commands are...", LL_INFO);
 		Server->Log("write_mbr(mbr_filename,out_device)", LL_INFO);
 		Server->Log("get_clientnames", LL_INFO);
 		Server->Log("get_backupimages(restore_name)", LL_INFO);
+		Server->Log("get_backupimages_json(restore_name)", LL_INFO);
 		Server->Log("get_file_backups(restore_name)", LL_INFO);
 		Server->Log("download_mbr(restore_img_id,restore_time,restore_out)", LL_INFO);
 		Server->Log("download_image(restore_img_id,restore_time,restore_out)", LL_INFO);
 		Server->Log("download_files(restore_backupid,restore_time,restore_out)", LL_INFO);
 		Server->Log("download_progress(mbr_filename,out_device)", LL_INFO);
+		Server->Log("login(username,password)", LL_INFO);
+		Server->Log("read_mbr(device)", LL_INFO);
 		exit(0);
 	}
 	else if(cmd=="ping_server")
@@ -1167,7 +1324,8 @@ void do_restore(void)
 			}
 		}
 	}
-	else if(cmd=="get_backupimages" )
+	else if(cmd=="get_backupimages"
+		|| cmd == "get_backupimages_json")
 	{
 		tcpstack.Send(c, "GET BACKUPIMAGES "+Server->getServerParameter("restore_name")+"#pw="+pw);
 		std::string r=getResponse(c);
@@ -1185,7 +1343,14 @@ void do_restore(void)
 			}
 			else
 			{
-				std::cout << r.substr(1) ;
+				if (cmd == "get_backupimages_json")
+				{
+					std::cout << backup_images_output_to_json(r.substr(1));
+				}
+				else
+				{
+					std::cout << r.substr(1);
+				}
 				Server->destroy(c);exit(0);return;
 			}
 		}
@@ -1219,8 +1384,18 @@ void do_restore(void)
 		if(cmd=="download_mbr")
 			mbr=true;
 
+		LoginData login_data;
+		login_data.has_login_data = false;
+
+		std::string restore_token = Server->getServerParameter("restore_token");
+		if (!restore_token.empty())
+		{
+			login_data.has_login_data = true;
+			login_data.token = restore_token;
+		}
+
 		DownloadStatus dl_status;
-		int ec=downloadImage(atoi(Server->getServerParameter("restore_img_id").c_str()), Server->getServerParameter("restore_time"), Server->getServerParameter("restore_out"), mbr, LoginData(), dl_status);
+		int ec=downloadImage(atoi(Server->getServerParameter("restore_img_id").c_str()), Server->getServerParameter("restore_time"), Server->getServerParameter("restore_out"), mbr, login_data, dl_status);
 		exit(ec);
 	}
 	else if(cmd=="download_files")
@@ -1230,6 +1405,8 @@ void do_restore(void)
 	}
 	else if(cmd=="download_progress")
 	{
+		std::string decorate = Server->getServerParameter("decorate");
+
 		tcpstack.Send(c, "GET DOWNLOADPROGRESS#pw="+pw);
 		int lpc=0;
 		while(true)
@@ -1244,7 +1421,10 @@ void do_restore(void)
 					int npc=atoi(trim2(l).c_str());
 					if(npc!=lpc)
 					{
-					    std::cout << npc << std::endl;
+						if(decorate.empty())
+							std::cout << npc << std::endl;
+						else
+							std::cout << "Image restore complete: "<< npc << "%" << std::endl;
 					    lpc=npc;
 					}
 				}
@@ -1254,7 +1434,10 @@ void do_restore(void)
 		}
 		if(lpc!=100)
 		{
-		    std::cout << "100" << std::endl;
+			if (decorate.empty())
+			    std::cout << "100" << std::endl;
+			else
+				std::cout << "Image restore complete: 100%" << std::endl;
 		}
 
 		exit(0);
